@@ -1,31 +1,32 @@
 """
 "polymorphic" associations, ala SQLAlchemy.
 
-This example generalizes the function in poly_assoc_pk.py into a 
+This example generalizes the function in poly_assoc_pk.py into a
 function "association" which creates a new polymorphic association
 "interface".
 """
 
-from sqlalchemy import *
+from sqlalchemy import MetaData, Table, Column, Integer, String, ForeignKey
+from sqlalchemy.orm import mapper, relation, create_session, class_mapper
 
-metadata = MetaData('sqlite://', echo=False)
+metadata = MetaData('sqlite://')
 
 def association(cls, table):
     """create an association 'interface'."""
-    
+
     interface_name = table.name
     attr_name = "%s_rel" % interface_name
 
     metadata = table.metadata
-    association_table = Table("%s_associations" % interface_name, metadata, 
+    association_table = Table("%s_associations" % interface_name, metadata,
         Column('assoc_id', Integer, primary_key=True),
         Column('type', String(50), nullable=False)
     )
-    
+
     class GenericAssoc(object):
         def __init__(self, name):
             self.type = name
-    
+
     def interface(cls, name, uselist=True):
 
         mapper = class_mapper(cls)
@@ -48,20 +49,20 @@ def association(cls, table):
                     setattr(self, attr_name, GenericAssoc(table.name))
                 getattr(self, attr_name).targets = [value]
             setattr(cls, name, property(get, set))
-        
+
     setattr(cls, 'member', property(lambda self: getattr(self.association, '_backref_%s' % self.association.type)))
-    
+
     mapper(GenericAssoc, association_table, properties={
         'targets':relation(cls, backref='association'),
     })
-    
+
     return interface
 
 
 #######
 # addresses table
 
-addresses = Table("addresses", metadata, 
+addresses = Table("addresses", metadata,
     Column('id', Integer, primary_key=True),
     Column('assoc_id', None, ForeignKey('addresses_associations.assoc_id')),
     Column('street', String(100)),
@@ -81,12 +82,12 @@ mapper(Address, addresses)
 ######
 # sample # 1, users
 
-users = Table("users", metadata, 
+users = Table("users", metadata,
     Column('id', Integer, primary_key=True),
     Column('name', String(50), nullable=False),
     Column('assoc_id', None, ForeignKey('addresses_associations.assoc_id'))
     )
-    
+
 class User(object):
     pass
 
@@ -98,12 +99,12 @@ addressable(User, 'addresses', uselist=True)
 ######
 # sample # 2, orders
 
-orders = Table("orders", metadata, 
+orders = Table("orders", metadata,
     Column('id', Integer, primary_key=True),
     Column('description', String(50), nullable=False),
     Column('assoc_id', None, ForeignKey('addresses_associations.assoc_id'))
     )
-    
+
 class Order(object):
     pass
 
@@ -132,21 +133,21 @@ o1.address = Address()
 o1.address.street = '444 park ave.'
 
 sess = create_session()
-sess.save(u1)
-sess.save(o1)
+sess.add(u1)
+sess.add(o1)
 sess.flush()
 
-sess.clear()
+sess.expunge_all()
 
 # query objects, get their addresses
 
-bob = sess.query(User).get_by(name='bob')
+bob = sess.query(User).filter_by(name='bob').one()
 assert [s.street for s in bob.addresses] == ['123 anywhere street', '345 orchard ave']
 
-order = sess.query(Order).get_by(description='order 1')
+order = sess.query(Order).filter_by(description='order 1').one()
 assert order.address.street == '444 park ave.'
 
 # query from Address to members
 
-for address in sess.query(Address).list():
+for address in sess.query(Address).all():
     print "Street", address.street, "Member", address.member
